// Oleg Petruny proprietary.


#include "AgeOfWarUnit.h"

#include "Components/StaticMeshComponent.h"

#include "AgeOfWarManager.h"
#include <Components/BoxComponent.h>

const FAgeOfWarUnitStats& AAgeOfWarUnit::GetStats()
{
    if(!stats)
        stats = &AAgeOfWarManager::UnitGetStats(GetClass());

    return *stats;
}

void AAgeOfWarUnit::UpdateStats()
{
    health = GetStats().health;
}

void AAgeOfWarUnit::BeginPlay()
{
    if(manager)
    {
        UpdateStats();
    }

    Super::BeginPlay();
}

void AAgeOfWarUnit::EndPlay(const EEndPlayReason::Type EndPlayReason)
{
    Super::EndPlay(EndPlayReason);
}

AAgeOfWarUnit::AAgeOfWarUnit()
{
    //auto root = CreateDefaultSubobject<USceneComponent>(TEXT("Scene"));
    //
    //collision = CreateDefaultSubobject<UBoxComponent>(TEXT("Collision"));
    //collision->SetCollisionProfileName(TEXT("AgeOfWarUnit"));

    PrimaryActorTick.bCanEverTick = true;
    PrimaryActorTick.bStartWithTickEnabled = true;
}

void AAgeOfWarUnit::Tick(float deltaTime)
{
    Super::Tick(deltaTime);

    if(!manager || GetStats().moveSpeed <= 0)
        return;

    // move unit forward
    auto moveStep = GetActorForwardVector() * GetStats().moveSpeed;
    FHitResult hit;
    this->AddActorWorldOffset(moveStep, true, &hit, ETeleportType::None);

    // check if blocked
    if(!hit.bBlockingHit)
        return;

    forwardUnit = Cast<AAgeOfWarUnit>(hit.GetActor());
    if(!forwardUnit) // something is wrong, only block with units
    {
        Destroy();
        return;
    }

    // iterate to attackable unit
    for(int i = 1; forwardUnit->team == team && i < GetStats().attackDistance; i += forwardUnit->GetStats().unitSize)
        forwardUnit = forwardUnit->forwardUnit;

    // if forward unit is in the same team skip attack
    if(forwardUnit->team == team)
        return;

    // attack the forward unit
    if(lastAttackTimestamp + GetStats().attackRate - FPlatformTime::Seconds() < 0)
    {
        lastAttackTimestamp = FPlatformTime::Seconds();
        forwardUnit->Damage(GetStats().attackDamage);
    }
    GEngine->AddOnScreenDebugMessage(4, 5.0f, FColor::Yellow, FString::Printf(TEXT("Time %f"), lastAttackTimestamp + GetStats().attackRate - FPlatformTime::Seconds()));
}

void AAgeOfWarUnit::Damage(const int32 damage)
{
    GEngine->AddOnScreenDebugMessage(-1, 5.0f, FColor::Yellow, FString::Printf(TEXT("Damage %d taken by %s"), damage, *GetClass()->GetName()));

    health -= damage;

    if(health > 0)
        return;

    if(IsValid(manager))
        manager->OnUnitKill(this);
    else
        Destroy();
}
